/***************************************************************************
 *             __________               __   ___.
 *   Open      \______   \ ____   ____ |  | _\_ |__   _______  ___
 *   Source     |       _//  _ \_/ ___\|  |/ /| __ \ /  _ \  \/  /
 *   Jukebox    |    |   (  <_> )  \___|    < | \_\ (  <_> > <  <
 *   Firmware   |____|_  /\____/ \___  >__|_ \|___  /\____/__/\_ \
 *                     \/            \/     \/    \/            \/
 *
 * sdltest.c: Main function for SDL_TEST version.
 *
 * Copyright (C) 2006 Katie Stafford
 *
 * All files in this archive are subject to the GNU General Public License.
 * See the file COPYING in the source tree root for full license agreement.
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied.
 *
 ****************************************************************************/


#include "common.h"
#include "graphics.h"
#include "board.h"
#include "actor.h"
#include "player.h"
#include "zlib/zlib.h"
#include "imath.h"
#include "tag.h"

int cframe = 0;

void statusmsg(int time, const char *fmt, ...);

void debugf(const char *fmt, ...) {
    char buf[64];
    va_list lst;
    va_start(lst, fmt);
    RB vsnprintf(buf, 64, fmt, lst);
    va_end(lst);

    printf("%s\n", buf);
}

void statusmsg(int time, const char *fmt, ...) {
    char buf[128];
    va_list lst;
    va_start(lst, fmt);
    vsnprintf(buf, 128, fmt, lst);
    va_end(lst);

    printf("msg(%d): %s\n", time, buf);
}

void blend(Uint8 * dst, Uint8 * src, int len, int amt, int trs) {
    int i;
    for (i = 0; i < len; i++) {
	int s = *src++;
	int d = *dst;
	int v = ((d) * (256 - amt) + (s) * (amt)) >> 8;
	if (abs(s - d) < trs)
	    v = s;
	*dst++ = v;
    }
}
void setpal(SDL_Surface * scr) {
    SDL_Color colors[256];
    SDL_Color *c = colors;
    int i;

    for (i = 0; i < 256; c++, i++) {
	int v = 255 - i;
	c->r = 45 * v / 255;
	c->g = 180 * v / 255;
	c->b = v;
    }
    SDL_SetPalette(scr, SDL_LOGPAL | SDL_PHYSPAL, colors, 0, 256);
}

#define INTERP 6
void ts_enum_print(tag t, void* data) {
    printf("%s\n",tag_name(t));
}

int main(int argc, char **argv) {
    Uint8 *src1, *dest, *tmpsurf, *screenbuf;
    int done = 0;
    int press;
    int x, y, j, k, l, fv;
    int nexttick;
    int ci = 0;
    int fvel = 0, avel = 0, svel = 0, zvel = 0;
    int fire = 0;


    SDL_Surface *screen;

    SDL_Init(SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE);

    screen =
	SDL_SetVideoMode(XDIM * MAG, YDIM * MAG, 8,
			 SDL_HWPALETTE | SDL_DOUBLEBUF);
    if (!screen) {
	printf("Couldn't set video mode: %s\n", SDL_GetError());
	exit(-1);
    }

    setpal(screen);

    tmpsurf = malloc(XDIM * YDIM / 4);

    screenbuf = malloc(XDIM * YDIM * MAG * MAG);
    memset(screenbuf, 0, XDIM * YDIM * MAG * MAG);

    loadtex("rocken.tex");
    int res = loadboard("demoboard.rbd");
    if (res) {
	printf("Couldn't load board (%d)\n", res);
	exit(-1);

    }

    //ADDTAG(cplayer, is_invulnerable);
    ADDTAG(cplayer, has_key);


    fv = 0;
    nexttick = SDL_GetTicks();
    while (!done) {
	SDL_Event evt;
	while (SDL_PollEvent(&evt)) {
	    switch (evt.type) {
	    case SDL_KEYDOWN:
		switch (evt.key.keysym.sym) {
		case SDLK_p:
                    printf("save!\n");
                    save_game("test.sav", "test");
                    break;
		case SDLK_o:
                    printf("load!\n");
                    load_game("test.sav");
                    break;
		case SDLK_ESCAPE:
		    done = 1;
		    break;
		default:
		    break;
		}
	    case SDL_KEYUP:
		press = evt.type == SDL_KEYDOWN;
		switch (evt.key.keysym.sym) {
		case SDLK_UP:
		    fvel = press;
		    break;
		case SDLK_DOWN:
		    fvel = -press;
		    break;
		case SDLK_LEFT:
		    avel = -press;
		    break;
		case SDLK_RIGHT:
		    avel = press;
		    break;
		case SDLK_a:
		    zvel = -press;
		    break;
		case SDLK_z:
		    zvel = press;
		    break;
		case SDLK_COMMA:
		    svel = -press;
		    break;
		case SDLK_PERIOD:
		    svel = press;
		    break;
		case SDLK_SPACE:
		    fire = press;
		    break;
		case SDLK_w:
		    fv = press;
		    break;
		case SDLK_e:
		    fv = -press;
		    break;

		default:
		    break;
		}
	    default:
		break;

	    }
	}
	player_set_controls(cplayer,fvel, svel, zvel, avel, fire);

	ci++;
	if (ci == INTERP) {
	    board_changed();
	    ci = 0;
	    num_sprites = 0;
	    move_actors();

            int div = XDIM * ((YDIM/2) / 4);

	    memset(tmpsurf, 0x55, div);
	    memset(tmpsurf+div, 0xaa, (XDIM * YDIM / 4) - div);

            preparescreen(cplayer->x, cplayer->y, player_get_posz(cplayer), cplayer->ang,
                          0, YDIM, YDIM/2);

	    drawscreen(tmpsurf);
	    
	    draw_actors();
	    
	    drawsprites(tmpsurf);

	    src1 = tmpsurf;
	    dest = screenbuf;
	    for (y = 0; y < (YDIM / 4); y++) {
		for (x = 0; x < XDIM; x++) {
		    Uint8 s1 = *src1++;
		    for (l = 0; l < 4; l++) {
			Uint8 gs1 = (((s1 >> (l << 1)) & 3) * 255) / 4;
			for (j = 0; j < MAG; j++)
			    for (k = 0; k < MAG; k++)
				dest[x * MAG + j +
				     (XDIM * MAG) * (((y << 2) + l) * MAG +
						     k)] = gs1;
		    }
		}
	    }
	}

	blend(screen->pixels, screenbuf, XDIM * YDIM * MAG * MAG, 32, 9);

	src1 = findgoalbuf;
	dest = screen->pixels;
	for (x = 0; x < 64; x++) {
	    for (y = 0; y < 64; y++) {
		int v = *src1++;
		dest[x + (XDIM * MAG) * y] = v << 3;
	    }
	}
	SDL_Flip(screen);

	while (SDL_GetTicks() < nexttick) {
	    SDL_Delay(1);
	}
	nexttick += 80 / INTERP;
	if (SDL_GetTicks() > nexttick) {
	    nexttick = SDL_GetTicks();
	}


    }
    printf("%d\n", num_actors_allocated);
    clear_actors();
    printf("%d\n", num_actors_allocated);
    SDL_Quit();
    return 0;
}

