/***************************************************************************
 *             __________               __   ___.
 *   Open      \______   \ ____   ____ |  | _\_ |__   _______  ___
 *   Source     |       _//  _ \_/ ___\|  |/ /| __ \ /  _ \  \/  /
 *   Jukebox    |    |   (  <_> )  \___|    < | \_\ (  <_> > <  <
 *   Firmware   |____|_  /\____/ \___  >__|_ \|___  /\____/__/\_ \
 *                     \/            \/     \/    \/            \/
 *
 * util.c: Miscellaneous utilities
 *
 * Copyright (C) 2006 Katie Stafford
 *
 * All files in this archive are subject to the GNU General Public License.
 * See the file COPYING in the source tree root for full license agreement.
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied.
 *
 ****************************************************************************/

#include "util.h"

#ifdef SDLTEST
char lcd_framebuffer[SCREEN_SIZE];
#endif

objectgroup option_handlers;



uint32 get_interval_usec(timeprec_t* start, timeprec_t* end) {
    uint32 tickdiff;

#ifndef SDL_TEST
    tickdiff = (end->ticks - start->ticks)*10000;

#ifdef CPU_COLDFIRE
    tickdiff += (((int)end->tickfrac - (int)start->tickfrac) * 10000 ) / TRR0;
#endif

#else /* SDL_TEST */
    tickdiff = (end->ticks - start->ticks)*1000;
#endif

    return tickdiff;
}


#ifdef CPU_COLDFIRE
void mac_save_state(void* ptr) {

    asm(
        "move.l    %%macsr, %%d1                         \n"
        "move.l    %%d1, (%[ptr])+                       \n"
        "move.l    #0, %%macsr                           \n"
        "move.l    %%accext01, %%d0                      \n"
        "move.l    %%d0, (%[ptr])+                       \n"
        "move.l    %%accext23, %%d0                      \n"
        "move.l    %%d0, (%[ptr])+                       \n"
        "move.l    %%acc0, %%d0                          \n"
        "move.l    %%d0, (%[ptr])+                       \n"
        "move.l    %%acc1, %%d0                          \n"
        "move.l    %%d0, (%[ptr])+                       \n"
        "move.l    %%acc2, %%d0                          \n"
        "move.l    %%d0, (%[ptr])+                       \n"
        "move.l    %%acc3, %%d0                          \n"
        "move.l    %%d0, (%[ptr])+                       \n"
        "move.l    %%d1, %%macsr                         \n"
        :
        : [ptr] "a" (ptr)
        : "d0", "d1", "memory");

}
void mac_restore_state(void* ptr) {
    asm(
        "move.l    (%[ptr])+, %%d1                       \n"
        "move.l    #0, %%macsr                           \n"
        "move.l    (%[ptr])+, %%d0                       \n"
        "move.l    %%d0, %%accext01                      \n"
        "move.l    (%[ptr])+, %%d0                       \n"
        "move.l    %%d0, %%accext23                      \n"
        "move.l    (%[ptr])+, %%d0                       \n"
        "move.l    %%d0, %%acc0                          \n"
        "move.l    (%[ptr])+, %%d0                       \n"
        "move.l    %%d0, %%acc1                          \n"
        "move.l    (%[ptr])+, %%d0                       \n"
        "move.l    %%d0, %%acc2                          \n"
        "move.l    (%[ptr])+, %%d0                       \n"
        "move.l    %%d0, %%acc3                          \n"
        "move.l    %%d1, %%macsr                         \n"
        :
        : [ptr] "a" (ptr)
        : "d0", "d1", "memory");

}
#endif

void* save_screen() {
    void* buffer = malloc(SCREEN_SIZE);
    RB memcpy(buffer, RB lcd_framebuffer, sizeof(lcd_framebuffer));
    return buffer;
}

void restore_screen(void* buffer) {
    RB memcpy(RB lcd_framebuffer, buffer, sizeof(lcd_framebuffer));
}

int load_settings(const char* path) {
    int num_options, i, ret;
    buffer buf;
    buf.data = NULL;
    buf.pos = 0;
    buf.len = 0;
    buf.cap = 0;
    ret = loadfile(path, &buf);

    if (ret < 0) {
        if (buf.data) free(buf.data);
        return ret;
    }
    
    num_options = readu16(&buf);
    for (i = 0; i < num_options; i++) {
        option_handler* h;
        int nextpos = readu16(&buf);
        nextpos += buf.pos;
        h = (option_handler*)read_namedobject(&option_handlers, &buf);
        if (h) {
            h->serial(&buf, PHASE_READ);
        }
        buf.pos = nextpos;
    }
    free(buf.data);
    return 0;
}

int save_settings(const char* path) {
    int i;
    buffer buf;
    buf.cap = 128;
    buf.len = 0;
    buf.pos = 0;
    buf.data = malloc(buf.cap);

    writeu16(&buf, option_handlers.size);
    for (i = 0; i < option_handlers.size; i++) {
        option_handler* h = (option_handler*)option_handlers.objects[i];
        int opos = buf.pos;
        int tpos;
        writeu16(&buf, 0);
        writestr(&buf, h->name);
        h->serial(&buf, PHASE_WRITE);
        tpos = buf.pos;
        buf.pos = opos;
        writeu16(&buf, tpos - opos - 2);
        buf.pos = tpos;
    }
    int ret = writefile(path, &buf);

    free(buf.data);
    
    return ret;
    
}

void _register_namedobject(objectgroup* grp, void* obj) {
    if (!grp->size) {
	grp->objects = malloc(16*sizeof(namedobject*));
    } else if ((grp->size & 15) == 0) {
	grp->objects = realloc(grp->objects, (grp->size+16)*sizeof(namedobject*));
    }
    grp->objects[grp->size++] = (namedobject*)obj;
}

namedobject* find_namedobject(objectgroup* grp, const char* name) {
    int i;
    for (i = 0; i < grp->size; i++) {
	if (!RB strcmp(grp->objects[i]->name, name))
	    return grp->objects[i];
    }
    return NULL;
}

namedobject* read_namedobject(objectgroup* grp, buffer* b) {
    char name[64];
    readstr(b, name, 64);
    return find_namedobject(grp, name);
}
